'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.API = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      *  Handle methods for creating APIs
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */


var _mongoose = require('mongoose');

var _mongoose2 = _interopRequireDefault(_mongoose);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Schema = _mongoose2.default.Schema;


/*
 *  Some generic errors
 */
var ENO_MODEL = { code: 404, message: 'No model set on API' };
var EINVALID_ID = { code: 400, message: 'Invalid ID' };

var API = exports.API = function () {
  function API(mighty, config) {
    var _this2 = this;

    _classCallCheck(this, API);

    this.globalConfig = mighty.config;
    this.app = mighty.app;
    this.config = config;
    this.model = null;

    if (this.config.model) {
      var s = new Schema(this.config.model.schema, this.config.model.options);
      if (this.config.model.middleware && _typeof(this.config.model.middleware) === 'object') {
        for (var type in this.config.model.middleware) {
          var m = this.config.model.middleware[type];
          if (m instanceof Array) {
            for (var i = 0, len = m.length; i < len; i++) {
              s[type](m[i].type, m[i].fn);
            }
          } else {
            s[type](m.type, m.fn);
          }
        }
      }
      this.model = _mongoose2.default.model(this.config.model.name, s);
      console.log('Setup model: ' + this.config.model.name + ' in [' + this.config.data_space + ']');
    }

    this.handlers = {
      get: function get(req, res, next) {
        // example, need to consolidate this
        var _this = _this2;
        if (!_this.model) {
          API.handleError(res, ENO_MODEL);
          return next();
        }

        var skip = 0;
        var limit = 50;
        var sortDir = 1;
        var sort = { 'create_time': sortDir };

        var conditions = _utils2.default.convertParamToObject(req.params.conditions);
        var fields = _utils2.default.convertParamToObject(req.params.fields);
        req.params.not = _utils2.default.convertParamToObject(req.params.not);
        var search = _utils2.default.convertParamToObject(req.params.search);

        if (req.params.sortDir) {
          if (req.params.sort.charAt(0) === '[' || req.params.sort.charAt(0) === '{') {
            try {
              sortDir = JSON.parse(req.params.sortDir);
            } catch (err) {
              API.handleError(res, { code: 500, message: 'Invalid sortDir parameter' });
            }
          } else {
            sortDir = parseInt(req.params.sortDir, 10);
          }
        }

        if (req.params.sort) {
          if (req.params.sort.charAt(0) === '[' || req.params.sort.charAt(0) === '{') {
            try {
              sort = JSON.parse(req.params.sort);
            } catch (err) {
              API.handleError(res, { code: 500, message: 'Invalid sort parameter' });
            }
          } else {
            sort = {};
            sort[req.params.sort] = sortDir;
          }
        }

        if (req.params._id) {
          conditions._id = req.params._id;
        }

        if (conditions._id) {
          if (conditions._id instanceof Array) {
            var Ids = { $in: [] };
            for (var i = 0; i < conditions._id.length; i++) {
              try {
                Ids['$in'].push(_mongoose2.default.mongo.ObjectId(conditions._id[i]));
              } catch (e) {
                return API.handleError(res, { code: 400, message: e.message });
              }
            }
            conditions._id = Ids;
          } else if (_typeof(conditions._id) === 'object') {
            for (var i in conditions._id) {
              if (conditions._id[i] instanceof Array) {
                for (var j = 0; j < conditions._id[i].length; j++) {
                  try {
                    conditions._id[i][j] = _mongoose2.default.mongo.ObjectId(conditions._id[i][j]);
                  } catch (e) {
                    return API.handleError(res, { code: 400, message: e.message });
                  }
                }
              } else {
                try {
                  conditions._id[i] = _mongoose2.default.mongo.ObjectId(conditions._id[i]);
                } catch (e) {
                  return API.handleError(res, { code: 400, message: 'Invalid ID' });
                }
              }
            }
          } else {
            try {
              conditions._id = _mongoose2.default.mongo.ObjectId(conditions._id);
            } catch (e) {
              return API.handleError(res, { code: 400, message: 'Invalid ID' });
            }
          }
        }

        var method = 'find';

        if (conditions._id) {
          method = 'findOne';
        }

        var q = _this.model[method](conditions);
        if (fields) {
          q.select(fields);
        }
        q.limit(limit).skip(skip).sort(sort).then(function (results) {
          res.send(200, results);
          return next();
        }).catch(function (err) {
          API.handleError(res, { code: 500, message: err });
          return next();
        });
      },
      post: function post(req, res, next) {
        var _this = _this2;
        if (!_this.model) {
          API.handleError(res, ENO_MODEL);
          return next();
        }

        var m = new _this.model(req.params);
        m.save().then(function (result) {
          if (req.route.config && req.route.config.afterConditions) {
            _utils2.default.handleAfterConditions(result, req.route.config.afterConditions, req).then(function (result) {
              res.send(200, result);
              return next();
            }).catch(function (err) {
              API.handleError(res, { code: 500, message: err });
              return next();
            });
          } else {
            res.send(200, result);
            return next();
          }
        }).catch(function (err) {
          API.handleError(res, { code: 500, message: err });
          return next();
        });
      },
      put: function put(req, res, next) {
        var _this = _this2;
        if (!_this.model) {
          API.handleError(res, ENO_MODEL);
          return next();
        }
        if (!req.params._id) {
          API.handleError(res, { code: 404, message: 'Missing _id' });
          return next();
        }

        req.params.conditions = _this2.model.findOneAndUpdate(req.params.conditions, req.params, { new: true }).then(function (result) {
          if (result === null) {
            API.handleError(res, { code: 404, message: _this.config.model.name + ' not found!' });
            return next();
          }

          res.send(200, result);
          return next();
        }).catch(function (err) {
          API.handleError(res, { code: 500, message: err });
          return next();
        });
      },
      del: function del(req, res, next) {
        var _this = _this2;
        if (!_this.model) {
          API.handleError(res, ENO_MODEL);
          return next();
        }

        if (!req.params._id) {
          API.handleError(res, { code: 404, message: 'Missing _id' });
          return next();
        }

        _this2.model.findOneAndRemove(req.params._id).then(function (result) {
          if (result === null) {
            API.handleError(res, { code: 404, message: _this.config.model.name + ' not found!' });
            return next();
          }

          res.send(200, result);
          return next();
        }).catch(function (err) {
          API.handleError(res, { code: 500, message: err });
          return next();
        });
      }
    };
  }

  /*
   * Handle an API error
   *
   * @param {object} res - The result
   * @param {object} error - The error object
   * @param {number} error.code - The status code to be returned.
   * @param {string} error.message - The message to be returned.
   */


  _createClass(API, [{
    key: 'loadAPI',


    /*
     * Load an API with a configuration
     *
     * @param {object} config - The configuration for the API
     */
    value: function loadAPI() {
      var config = this.config;
      var app = this.app;
      try {
        if (config.api) {
          for (var method in config.api) {
            // setup admin routes if necessary, overrides 'enabled' for standard CRUD
            if (config.api[method].admin) {
              if (!config.api[method].admin_prefix) {
                config.api[method].admin_prefix = 'admin';
              }
              var _path = '/' + config.api[method].admin_prefix + '/' + config.data_space;
              if (config.route_space) {
                _path = '/admin' + config.route_space;
              }
              if (method === 'put' || method === 'del') {
                _path += '/:_id';
              }
              API.createRoute(this, _path, method);
            }

            if (!config.api[method].enabled) continue;

            var path = '/' + config.data_space;
            if (config.route_space) {
              path = config.route_space;
            }
            if (method === 'put' || method === 'del') {
              path += '/:_id';
            }
            API.createRoute(this, path, method);
          }
        }

        if (config.routes) {
          for (var _path2 in config.routes) {
            var _method = config.routes[_path2].method.toLowerCase();
            app[_method]({
              path: _path2,
              version: config.version,
              name: config.name,
              config: config.routes[_path2]
            }, config.routes[_path2].controller);
            console.log('Setup route: [' + _method.toUpperCase() + '] ' + _path2);
          }
        }
      } catch (err) {
        console.log(err);
        return null;
      }

      return this;
    }
  }], [{
    key: 'handleError',
    value: function handleError(res, error) {
      return res.json(error.code, error);
    }
  }, {
    key: 'createRoute',
    value: function createRoute(context, path, method) {
      var verbose = arguments.length <= 3 || arguments[3] === undefined ? true : arguments[3];

      var app = context.app;
      var config = Object.assign({}, context.config);
      app[method]({
        path: path,
        version: config.version,
        name: config.name,
        config: config.api[method]
      }, context.handlers[method]);
      if (verbose) {
        console.log('Setup route: [' + method.toUpperCase() + '] ' + path);
      }

      if (method === 'get') {
        path += '/:_id';
        app[method]({
          path: path,
          version: config.version,
          name: config.name,
          config: config.api[method]
        }, context.handlers[method]);
        if (verbose) {
          console.log('Setup route: [' + method.toUpperCase() + '] ' + path);
        }
      }
    }
  }]);

  return API;
}();