'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
var Schema = require('mongoose').Schema;
var nodeRSA = require('node-rsa');

var generateLicenseKey = function generateLicenseKey(level) {
  if (!level) {
    level = 2048;
  }

  var key = new nodeRSA({ b: level });
  var publicKeyParts = key.exportKey('pkcs1-public-pem').split('\n');
  var publicKey = '';
  for (var i = 1; i < publicKeyParts.length - 1; i++) {
    publicKey += publicKeyParts[i];
  }
  return { private: key.exportKey('pkcs1-private-pem'), public: publicKey };
};

var config = {
  module_name: 'License',
  data_space: 'license',
  route_space: '/license',
  version: '1.0.0',
  dependencies: {
    // 'instrmnt.other_module': 'x.x.x' -- use the data_space
  },
  model: {
    // Mongoose model
    name: 'License',
    schema: {
      _organization: { type: Schema.Types.ObjectId, ref: 'Organization' },
      template: { type: Schema.Types.ObjectId, ref: 'LicenseTemplate' },
      keys: { type: Object, default: generateLicenseKey },
      expires: { type: Date },
      auto_renew: { type: Boolean, default: true },
      next_charge: { type: Date },
      server_id: { type: String },
      server_ip: { type: String },
      user_seats: { type: Number },
      balance: { type: Number, default: 0 },
      invoices: [{ type: Schema.Types.ObjectId, ref: 'LicenseInvoice' }],
      billing_details: { type: Schema.Types.ObjectId, ref: 'PaymentMethod' },
      license_agreement: { type: String }
    }
  },
  api: {
    get: {
      enabled: true,
      admin: true
      // jsapi: false,
      // jsapi_authentication: false,
      // socket: false,
      // beforeConditions: {},
      // afterConditions: {}
    },
    post: {
      enabled: false,
      admin: true
    },
    put: {
      enabled: false,
      admin: true
    },
    del: {
      enabled: false,
      admin: true
    }
  }
};

exports.default = config;