'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _utils = require('../../utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Schema = require('mongoose').Schema;


var config = {
  module_name: 'Registry',
  data_space: 'registry',
  route_space: '/registry',
  version: '1.0.0',
  dependencies: {
    // 'instrmnt.other_module': 'x.x.x' -- use the data_space
  },
  model: {
    // Mongoose model
    name: 'Registration',
    schema: {
      _organizations: [{ type: Schema.Types.ObjectId, ref: 'Organization' }],
      _owners: [{ type: Schema.Types.ObjectId, ref: 'User' }],
      app_name: { type: String },
      app_repository: { type: String }, // the git repo where it is hosted
      app_repository_credentials: {
        hostname: { type: String },
        username: { type: String },
        access_token: { type: String }
      }, // optional credentials for accessing private repos.
      app_location: { type: String }, // IP where it is hosted
      hostname: { type: String }, // optional hostname where app is hosted
      app_key: { type: String, select: false, default: _utils2.default.generateKey },
      usage: {}, // stats we can implement later
      license_type: { type: String, enum: ['free', 'monthly', 'single'] }
    }
  },
  api: {
    get: {
      enabled: true,
      authentication: true,
      organization: true
    },
    post: {
      enabled: false
    },
    put: {
      enabled: false
    },
    del: {
      enabled: false
    }
  }
};

exports.default = config;