'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _utils = require('../../utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Schema = require('mongoose').Schema;


var config = {
  module_name: 'Users',
  data_space: 'users',
  route_space: '/users',
  version: '1.0.0',
  dependencies: {
    // 'instrmnt.other_module': 'x.x.x' -- use the data_space
  },
  model: {
    // Mongoose model
    name: 'User',
    schema: {
      _owners: [{ type: Schema.Types.ObjectId, ref: 'User' }],
      name: { type: String },
      email: { type: String, required: true, unique: true },
      authentication_key: { type: String, select: false, default: null },
      secret: { type: String, select: false },
      password: { type: String, select: false },
      reset_code: { type: String, select: false },
      organizations: [{ type: Schema.Types.ObjectId, ref: 'Organization' }],
      default_organization: { type: Schema.Types.ObjectId, ref: 'Organization' },
      organization_profiles: [{
        permissions: [{ type: String }]
      }],
      avatar_url: { type: String },
      active: { type: Boolean, default: false },
      verified: { type: Boolean, default: false },
      verification_code: { type: String, select: false, default: _utils2.default.generateKey },
      last_login: { type: Date, default: new Date() },
      role: { type: String, default: 'user', enum: ['user', 'admin'] }
    },
    options: {
      'toJSON': {
        transform: function transform(doc, ret, options) {
          delete ret.password;
          delete ret.secret;
          delete ret.reset_code;
          delete ret.authentication_key;
          delete ret.verification_code;
          delete ret._owners;
        }
      }
    }
  },
  api: {
    get: {
      enabled: false,
      admin: true
      // jsapi: false,
      // jsapi_authentication: false,
      // socket: false,
      // beforeConditions: {},
      // afterConditions: {}
    },
    post: {
      enabled: false,
      admin: true
    },
    put: {
      enabled: true,
      authentication: true,
      owner: true,
      admin: true
    },
    del: {
      enabled: false,
      admin: true
    }
  }
};

exports.default = config;